function []=td_plot(res)
% PURPOSE: Generate graphic output of temporal disaggregation methods
% ------------------------------------------------------------
% SYNTAX: td_plot(res);
% ------------------------------------------------------------
% OUTPUT: Detailed grahic output:
%         - Objective function vs. rho
%         - Estimate and one-sigma limits
%         - High frequency conformity: estimate and indicators (levels)
%         - Low frequency series vs low frequency indicators (scaled)
%         - High frequency conformity: yoy rates of growth
%         - High frequency conformity: yoy rates of growth (last periods)
%         - High and low frequency series: y vs Y
%         - High and low frequency residuals: u vs U
% ------------------------------------------------------------
% INPUT: res: structure generated by td programs
% ------------------------------------------------------------
% LIBRARY: tasa
% ------------------------------------------------------------
% SEE ALSO: chowlin, fernandez, litterman, td_plot

% written by:
% Enrique M. Quilis
% Instituto Nacional de Estadistica
% Paseo de la Castellana, 183
% 28046 - Madrid (SPAIN)

% -----------------------------------------------------------
% -----------------------------------------------------------
% Loading the structure

meth=res.meth;

% -----------------------------------------------------------
% Basic parameters 
N=res.N;
n=res.n;
pred=res.pred;
s=res.s;
p=res.p;
ta=res.ta;
type=res.type;

% -----------------------------------------------------------
% Series
Y = res.Y;
x = res.x;
y    = res.y;
d_y  = res.y_dt;
y_li = res.y_lo;
y_ls = res.y_up;

% -----------------------------------------------------------
% Residuals
u = res.u;
U = res.U;

% -----------------------------------------------------------
% Parameters
beta = res.beta;
beta_sd =res.beta_sd;
beta_t =res.beta_t;
rho = res.rho;

% -----------------------------------------------------------
% Information criteria
aic=res.aic;
bic=res.bic;

% -----------------------------------------------------------
% -----------------------------------------------------------
% Graphic output

% -----------------------------------------------------------
% Objective function vs. rho

switch res.meth
case {'Fernandez'}
   % Do nothing
case {'Chow-Lin','Litterman'}
   switch type
   case 0
      plot(res.r,-res.val,'ro',res.r,-res.val);
      title  ('Weighted least squares vs innovational parameter');
      ylabel ('weighted least squares');
   case 1
      plot(res.r,res.val,'ro',res.r,res.val);
      title  ('Likelihood vs innovational parameter');
      ylabel ('likelihood');
   end; % of switch ml
   grid on;
   xlabel ('innovational parameter');
end

% -----------------------------------------------------------
% Estimate and one-sigma limits

figure;
t=1:n;
plot(t,y,'r-',t,y_li,'b-',t,y_ls,'b-');
grid off; legend ('y','y-sigma','y+sigma',0);
title ('High frequency estimate +/- sigma ');
xlabel ('time');

% -----------------------------------------------------------
% High frequency conformity: estimate and indicators (levels)

figure;
xb=x*beta;
t=1:n;
plot(t,y,'r-',t,xb,'b-');
grid off; legend ('y','x*beta',0);
title ('High frequency conformity');
xlabel ('time');

% -----------------------------------------------------------
% Only if there is only one indicator (without intercept)
if (p == 2) 
   % -----------------------------------------------------------
   % High frequency conformity: yoy rates of growth
   figure;
   ty=tasa(y,s);  % Computing yoy rates of growth
   tx=tasa(x(:,2),s);
   t=1:n;
   plot(t,ty,'r-',t,tx,'b-');
   grid off; legend ('y','x',0);
   title ('High frequency conformity: yoy rates of growth');
   xlabel ('time');
   % -----------------------------------------------------------
   % High frequency conformity: yoy rates of growth
   % of last 3*s data
   figure;
   ty_li=tasa(y_li,s);
   ty_ls=tasa(y_ls,s);
   t=n-3*s:n;
   plot(t,ty(t),'r-o',t,ty_ls(t),'r:',t,ty_li(t),'r:',t,tx(t),'b-');
   grid off; legend ('y','y+sigma','y-sigma','x',0);
   title ('High frequency conformity: yoy rates of growth (detail)');
   xlabel ('time');
end;   

% -----------------------------------------------------------
% Low frequency series vs low frequency indicators (scaled)

C=aggreg(ta,N,s); 
X=C*x(1:n-pred,:);  % Only low freq. data span is considered

figure;
t=1:N;
plot(t,Y,'ro-',t,X*beta,'b+-');
grid off; legend ('Y','X*beta',0);
title ('Low frequency conformity');
xlabel ('time'); 

% -----------------------------------------------------------
% High and low frequency series: y vs Y
%   1. Generate a high freq. time series with the low freq. span
%   2. Plot both series (low freq. divided by g1)

if (ta == 1)
   g1=s;
else
   g1=1;
end

i=1; t=1;
while (t<=N)   
   c=0;
   while (c<s)
      ya(i,1)=Y(t,1);
      c=c+1;
      i=i+1;
   end
   t=t+1;
end;

figure;
t=1:n-pred;
y_N=y(1:n-pred);  % Only the part of low freq. span is considered
plot(t,y_N,'r-',t,ya/g1,'b-');
grid off; legend (meth,'y naive',0);
title ('High frequency and low frequency series');
xlabel ('time'); 

% -----------------------------------------------------------
% High and low frequency residuals: u vs U
%   1. Generate a high freq. time series with the low freq. data
%   2. Plot both series (low freq. divides by g1)

i=1; t=1;
while (t<=N)
   c=0;
   while (c<s)
      ua(i,1)=U(t,1);
      c=c+1;
      i=i+1;
   end
   t=t+1;
end;

figure;
t=1:n-pred;
u_N=u(1:n-pred);   % Only the part of low freq. data
plot(t,u_N,'r-',t,ua/g1,'b-');
grid off; legend ('u','U',0);
title ('High frequency and low frequency residuals');
xlabel ('time'); 
